package com.gic.enterprise.service.outer;

import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.EnterpriseStatusTypeEnum;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.EnterpriseListDTO;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.service.EnterpriseInitService;
import com.gic.enterprise.service.EnterpriseService;
import com.gic.store.utils.valid.ValidUtil;

/**
 * @author guojx
 * @date 2019/7/16 4:33 PM
 */
@Service("enterpriseApiService")
public class EnterpriseApiServiceImpl implements EnterpriseApiService {
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private EnterpriseInitService enterpriseInitService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Integer> saveEnterprise(EnterpriseDTO enterpriseDTO, UserDTO userDTO) {
        //valid param
        ServiceResponse paramResult = ValidUtil.allCheckValidate(enterpriseDTO, EnterpriseDTO.SaveEnterpriseValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        if (enterpriseService.isRepeatEnterpriseName(enterpriseDTO.getEnterpriseName(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户名称不能重复");
        }
        if (enterpriseService.isRepeatCompanyName(enterpriseDTO.getCompanyName(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "公司名称不能重复");
        }

        enterpriseDTO.setCreateTime(new Date());
        enterpriseDTO.setUpdateTime(new Date());
        enterpriseDTO.setIsInitComplete(0);
        enterpriseDTO.setStatus(EnterpriseStatusTypeEnum.EFFECTIVE.getCode());

        int enterpriseId = enterpriseService.saveEnterprise(enterpriseDTO);
        //商户初始化配置信息生成
        enterpriseInitService.initEnterpriseConfigInfo(enterpriseId);

        userDTO.setEnterpriseId(enterpriseId);
        //超级管理员
        userDTO.setSuperAdmin(1);
        ServiceResponse userResult = userApiService.saveUser(userDTO);
        if (userResult.isSuccess()) {
            return ServiceResponse.success(enterpriseId);
        } else {
            throw new CommonException(userResult.getCode(), userResult.getMessage());
        }
    }

    @Override
    public ServiceResponse<Integer> editEnterprise(EnterpriseDTO enterpriseDTO) {
        //valid param
        ServiceResponse paramResult = ValidUtil.allCheckValidate(enterpriseDTO,
                EnterpriseDTO.SaveEnterpriseValid.class, EnterpriseDTO.EditEnterpriseValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        //商户ID
        int enterpriseId = enterpriseDTO.getEnterpriseId();
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "企业ID错误，查询不到数据");
        }
        if (enterpriseService.isRepeatEnterpriseName(enterpriseDTO.getEnterpriseName(), enterpriseId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户名称不能重复");
        }
        if (enterpriseService.isRepeatCompanyName(enterpriseDTO.getCompanyName(), enterpriseId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "公司名称不能重复");
        }

        enterpriseDTO.setUpdateTime(new Date());
        enterpriseService.editEnterprise(enterpriseDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<EnterpriseDTO> getEnterpriseById(Integer enterpriseId) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID输入有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(EnterpriseDTO.class, tabEnterprise));
    }

    @Override
    public ServiceResponse disableEnterprise(Integer enterpriseId) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID输入有误");
        }
        enterpriseService.changeEnterpriseStatus(enterpriseId, EnterpriseStatusTypeEnum.DISABLE.getCode());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse enableEnterprise(Integer enterpriseId) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID输入有误");
        }
        enterpriseService.changeEnterpriseStatus(enterpriseId, EnterpriseStatusTypeEnum.EFFECTIVE.getCode());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<EnterpriseListDTO>> listEnterprise(String search, Integer serviceVersion, Integer status,
                                                                   String orderBy, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page page = enterpriseService.listEnterprise(search, serviceVersion, status, orderBy, pageNum, pageSize);
        Page<EnterpriseListDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse initEnterprise(Integer enterpriseId) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID输入有误");
        }
        enterpriseService.initEnterprise(enterpriseId);
        return ServiceResponse.success();
    }

}
